package com.sk.newsplayer.service;

import android.app.DownloadManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.Cursor;
import android.net.Uri;
import android.os.Environment;

import com.sk.newsplayer.R;
import com.sk.newsplayer.util.LogUtils;


public class AudioDownloader {

	protected static final String TAG = AudioDownloader.class.getSimpleName();
	
	private long mEnqueue = 0;
	private Context mContext;
	private String mPath;
    private DownloadManager mManager;
    private DownloadListener mListener;
    
	public AudioDownloader(Context context) {
		this(context, null);
	}

	public AudioDownloader(Context context, DownloadListener downloadListener) {
		this.mContext = context;
		this.mListener = downloadListener;
		this.mManager = (DownloadManager)context.getSystemService(Context.DOWNLOAD_SERVICE);
	}
	
	public AudioDownloader setPath(String mPath) {
		this.mPath = mPath;
		return this;
	}

	public AudioDownloader setListener(DownloadListener listener) {
		this.mListener = listener;
		return this;
	}

	public void download(String url){
		if (mListener!=null)
			mListener.onStart();
		
		Uri uri = Uri.parse(url);
		
		Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS).mkdirs();
		
		String fileName = url.substring(url.lastIndexOf("/"));
		mEnqueue = mManager.enqueue(new DownloadManager.Request(uri)
				.setAllowedNetworkTypes(DownloadManager.Request.NETWORK_WIFI|DownloadManager.Request.NETWORK_MOBILE)
				.setAllowedOverRoaming(false)
				.setTitle("Download")
				.setDescription(url)
				.setDestinationInExternalPublicDir(Environment.DIRECTORY_DOWNLOADS, fileName)
			);
		
		IntentFilter filter = new IntentFilter();
		filter.addAction(DownloadManager.ACTION_DOWNLOAD_COMPLETE);
		mContext.registerReceiver(mDownloadComplete, filter);
	}
	
	private BroadcastReceiver mDownloadComplete = new BroadcastReceiver(){
		@Override
		public void onReceive(Context context, Intent intent) {
			
			int errorCode = 0;
			String value = null;
			
			String action = intent.getAction();
            if (!DownloadManager.ACTION_DOWNLOAD_COMPLETE.equals(action)) return;
			
			intent.getLongExtra(DownloadManager.EXTRA_DOWNLOAD_ID, 0);
			DownloadManager.Query query = new DownloadManager.Query();
			query.setFilterById(mEnqueue);
			Cursor c = mManager.query(query);
			if (c.moveToFirst()) {
				int columnIndex = c.getColumnIndex(DownloadManager.COLUMN_STATUS);
				
				switch (c.getInt(columnIndex)){
					//Get error reason
					case DownloadManager.STATUS_FAILED:
					case DownloadManager.STATUS_PAUSED:
						String reason = c.getString(c.getColumnIndex(DownloadManager.COLUMN_REASON));
						errorCode = Integer.parseInt(reason);
						if (errorCode == DownloadManager.ERROR_FILE_ALREADY_EXISTS) {
							LogUtils.e(TAG, "Download failed because the requested destination file already exists.");
							errorCode = 0;
							value = getDownloadedFile(c);
						} 
						
						break;
					case DownloadManager.STATUS_SUCCESSFUL:
						value = getDownloadedFile(c);
						break;
					default:
						break;
				}
			}
			
			if (mListener!=null) {
				if (errorCode!=0)
					value = getMessage(errorCode);
				
				mListener.onFinish(errorCode, value);
			}
		}
	};
	
	
	public String getMessage(int code){
		String message = "";
		
		switch (code){
			case DownloadManager.ERROR_DEVICE_NOT_FOUND:
				//message = mContext.getString(R.string.error_device_not_found);
				//LogUtils.e(TAG, message);
				break;
			case DownloadManager.ERROR_FILE_ERROR:
				//message = mContext.getString(R.string.error_file_error);
				//LogUtils.e(TAG, message);
				break;
			case DownloadManager.ERROR_HTTP_DATA_ERROR:
				//message = mContext.getString(R.string.error_http_data_error);
				//LogUtils.e(TAG, message);
				break;
			case DownloadManager.ERROR_INSUFFICIENT_SPACE:
				//message = mContext.getString(R.string.error_insufficient_space);
				//LogUtils.e(TAG, message);
				break;
			case DownloadManager.ERROR_TOO_MANY_REDIRECTS:
				//message = mContext.getString(R.string.error_too_many_redirects);
				//LogUtils.e(TAG, message);
				break;
			case DownloadManager.ERROR_UNHANDLED_HTTP_CODE:
				//message = mContext.getString(R.string.error_unhandled_http_code);
				//LogUtils.e(TAG, message);
				break;
			case DownloadManager.PAUSED_QUEUED_FOR_WIFI:
				//message = mContext.getString(R.string.paused_queued_for_wifi);
				//LogUtils.e(TAG, message);
				break;
			case DownloadManager.PAUSED_UNKNOWN:
				//message = mContext.getString(R.string.paused_unknown);
				//LogUtils.e(TAG, message);
				break;
			case DownloadManager.PAUSED_WAITING_FOR_NETWORK:
				//message = mContext.getString(R.string.paused_waiting_for_network);
				//LogUtils.e(TAG, message);
				break;
			case DownloadManager.PAUSED_WAITING_TO_RETRY:
				//message = mContext.getString(R.string.paused_waiting_to_retry);
				//LogUtils.e(TAG, message);
				break;
		}
		
		return message;
	}
	
	//how to read the file: 
	//File mFile = new File(Uri.parse(localUri).getPath());
	protected String getDownloadedFile(Cursor c){
		String localUri = c.getString(c.getColumnIndex(DownloadManager.COLUMN_LOCAL_URI));
		
        //String fileName = localUri.substring("file://".length());
        String fileName = Uri.parse(localUri).getPath();
        LogUtils.d(TAG, "Download Status Successful. fileName:" + fileName);
        
//        if (mListener!=null){
//        	LogUtils.d(TAG, "Call downloadCompleteListener.onComplete()...");
//        	mListener.onFinish(mEnqueue, fileName);
//        }
        
        return fileName;
	}

	public static interface DownloadListener{
		public void onStart();
		public void onFinish(int errorCode, String result);
	}
}
