/*   
\ * Copyright (C) 2011 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.sk.newsplayer.service;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnCompletionListener;
import android.media.MediaPlayer.OnErrorListener;
import android.media.MediaPlayer.OnPreparedListener;
import android.net.Uri;
import android.net.wifi.WifiManager;
import android.net.wifi.WifiManager.WifiLock;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.text.TextUtils;
import android.widget.Toast;

import com.sk.newsplayer.MainActivity;
import com.sk.newsplayer.NewsPlayActivity;
import com.sk.newsplayer.R;
import com.sk.newsplayer.model.NewsItem;
import com.sk.newsplayer.model.RepeatMode;
import com.sk.newsplayer.util.LogUtils;

/**
 * Service that handles media playback. This is the Service through which we perform all the media
 * handling in our application. Upon initialization, it starts a {@link MediaRetriever} to scan
 * the user's media. Then, it waits for Intents (which come from our main activity,
 * {@link MainActivity}, which signal the service to perform specific operations: Play, Pause,
 * Rewind, Skip, etc.
 */
public class AudioPlayService extends Service implements OnCompletionListener, OnPreparedListener,
                OnErrorListener, AudioFocusable {
	
	// The tag we put on debug messages
    private final static String TAG = AudioPlayService.class.getSimpleName();

	public static final String EXTRA_ITEM = "item";
	
    NotificationManager mNotificationManager;

    // our media player
    MediaPlayer mPlayer = null;

    // our AudioFocusHelper object, if it's available (it's available on SDK level >= 8)
    // If not available, this will be null. Always check for null before using!
    AudioFocusHelper mAudioFocusHelper = null;

    // indicates the state our service:
    public enum State {
    	Idle,
        Retrieving, // the MediaRetriever is retrieving music
        Stopped,    // media player is stopped and not prepared to play
        Preparing,  // media player is preparing...
        Prepared,
        Playing,    // playback active (media player ready!). (but the media player may actually be
                    // paused in this state if we don't have audio focus. But we stay in this state
                    // so that we know we have to resume playback once we get focus back)
        Paused      // playback paused (media player ready!)
    };

    State mState = State.Idle;

    // if in Retrieving mode, this flag indicates whether we should start playing immediately
    // when we are ready or not.
    boolean mStartPlayingAfterRetrieve = false;

    // if mStartPlayingAfterRetrieve is true, this variable indicates the URL that we should
    // start playing when we are ready. If null, we should play a random song from the device
    Uri mWhatToPlayAfterRetrieve = null;

//    enum PauseReason {
//        UserRequest,  // paused by user request
//        FocusLoss,    // paused because of audio focus loss
//    };
    
    // why did we pause? (only relevant if mState == State.Paused)
    //PauseReason mPauseReason = PauseReason.UserRequest;

    // do we have audio focus?
    public enum AudioFocus {
        NoFocusNoDuck,    // we don't have audio focus, and can't duck
        NoFocusCanDuck,   // we don't have focus, but can play at a low volume ("ducking")
        Focused           // we have full audio focus
    }
    
    AudioFocus mAudioFocus = AudioFocus.NoFocusNoDuck;
    RepeatMode mRepeatMode = RepeatMode.RepeatNone;

    // title of the song we are currently playing
    NewsItem mNewsItem = null;

    // whether the song we are playing is streaming from the network
    //boolean mIsStreaming = false;

    // Wifi lock that we hold when streaming files from the internet, in order to prevent the
    // device from shutting off the Wifi radio
    WifiLock mWifiLock;
    
    //int mPositionA, mPositionB = -1;
    private int mAbPosition[] = {-1, -1};
    private int mAbIndex = 0;

    public static final String EXTRA_STATE 		= "state";
	public static final String EXTRA_POSITION 	= "position";
	public static final String EXTRA_DURATION 	= "duration";
	public static final String EXTRA_TITLE 		= "title";

	// These are the Intent actions that we are prepared to handle. Notice that the fact these
    // constants exist in our class is a mere convenience: what really defines the actions our
    // service can handle are the <action> tags in the <intent-filters> tag for our service in
    // AndroidManifest.xml.
    public static final String ACTION_PLAY 				= "com.sk.newsplayer.action.PLAY";
    public static final String ACTION_PAUSE 			= "com.sk.newsplayer.action.PAUSE";
    public static final String ACTION_STOP 				= "com.sk.newsplayer.action.STOP";
    public static final String ACTION_FORWARD			= "com.sk.newsplayer.action.FORWARD";
    public static final String ACTION_REWARD 			= "com.sk.newsplayer.action.REWARD";
    public static final String ACTION_AB_REPEAT 		= "com.sk.newsplayer.action.AB_REPEAT";
    public static final String ACTION_REPEAT_MODE 		= "com.sk.newsplayer.action.REPEAT_MODE";
    public static final String ACTION_SEEK_TO  			= "com.sk.newsplayer.action.SEEK_TO";
    public static final String ACTION_PLAY_NEWS			= "com.sk.newsplayer.action.PLAY_NEWS";
    
    //ACTION for receiver
    public static final String ACTION_STATE_CHANGED 		= "com.sk.newsplayer.action.STATE_CHANGED";
    public static final String ACTION_POSITION_CHANGED 		= "com.sk.newsplayer.action.POSITION_CHANGED";
    public static final String ACTION_REPEAT_MODE_CHANGED 	= "com.sk.newsplayer.action.REPEAT_MODE_CHANGED";
    public static final String ACTION_NEXT_NEWS 			= "com.sk.newsplayer.action.NEXT_NEWS";

    public static final int UM_TRACK_AUDIO = 1001;
    
    // The volume we set the media player to when we lose audio focus, but are allowed to reduce
    // the volume instead of stopping playback.
    public final float DUCK_VOLUME = 0.1f;

    // The ID we use for the notification (the onscreen alert that appears at the notification
    // area at the top of the screen as an icon -- and as text as well if the user expands the
    // notification area).
    final int NOTIFICATION_ID = 1;

    // Our instance of our MusicRetriever, which handles scanning for media and
    // providing titles and URIs as we need.
    Notification mNotification = null;
   
	Handler mTrackHandler = new Handler(){
		@Override
		public void handleMessage(Message msg){
			if (mPlayer==null) return;
			
			LogUtils.d(TAG, "##### mHandler handleMessage....");
			
			if (isAbRepeat() && mPlayer.getCurrentPosition()>=mAbPosition[1]){
    			mPlayer.seekTo(mAbPosition[0]);
    		}
			
			if (mState == State.Prepared || mState == State.Playing){
		    	LogUtils.d(TAG, "==== MusicService sendBroadcast ACTION_POSITION_CHANGED:" + mPlayer.getCurrentPosition());
		    	Intent intent = new Intent(ACTION_POSITION_CHANGED);
		    	intent.putExtra(EXTRA_DURATION, mPlayer.getDuration());
		    	intent.putExtra(EXTRA_POSITION, mPlayer.getCurrentPosition());
		    	sendBroadcast(intent);
			}
			
		    if (mPlayer.isPlaying())
			   mTrackHandler.sendEmptyMessageDelayed(UM_TRACK_AUDIO, 1000);
		}
	};

    private synchronized void setState(State aState){
    	mState = aState;
    	LogUtils.d(TAG, "==== MusicService sendBroadcast State:" + mState);
    	Intent intent = new Intent(ACTION_STATE_CHANGED);
    	intent.putExtra(EXTRA_STATE, 	 mState.ordinal());
    	
    	int duration = 0;
    	if (aState == State.Prepared || mState == State.Playing)
    		duration = mPlayer.getDuration();

    	intent.putExtra(EXTRA_DURATION,  duration);
    	switch (aState){
    		case Prepared:
    			intent.putExtra(EXTRA_ITEM, mNewsItem);
    			break;
    	}
    	
    	sendBroadcast(intent);
    }
    
	/**
     * Makes sure the media player exists and has been reset. This will create the media player
     * if needed, or reset the existing media player if one already exists.
     */
    private void createMediaPlayerIfNeeded() {
        if (mPlayer == null) {
            mPlayer = new MediaPlayer();

            // Make sure the media player will acquire a wake-lock while playing. If we don't do
            // that, the CPU might go to sleep while the song is playing, causing playback to stop.
            //
            // Remember that to use this, we have to declare the android.permission.WAKE_LOCK
            // permission in AndroidManifest.xml.
            //***mPlayer.setWakeMode(getApplicationContext(), PowerManager.PARTIAL_WAKE_LOCK);

            // we want the media player to notify us when it's ready preparing, and when it's done. playing:
            mPlayer.setOnPreparedListener(this);
            mPlayer.setOnCompletionListener(this);
            mPlayer.setOnErrorListener(this);
        }
        else
            mPlayer.reset();
    }

    /**
     * Check if it is in ABRepeat mode
     * @return
     */
    protected boolean isAbRepeat() {
    	int a = mAbPosition[0];
    	int b = mAbPosition[1];
    	return (a>-1 && b>-1) && (b>a);
	}

	@Override
    public void onCreate() {
        LogUtils.i(TAG, "debug: Creating service");
        
        createMediaPlayerIfNeeded();
        setState(State.Idle);

        // Create the Wifi lock (this does not acquire the lock, this just creates it)
        mWifiLock = ((WifiManager) getSystemService(Context.WIFI_SERVICE))
                        .createWifiLock(WifiManager.WIFI_MODE_FULL, "mylock");

        mNotificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);

        // Create the retriever and start an asynchronous task that will prepare it.
        //(new PrepareNewsRetrieverTask(mRetriever, this)).execute();

        // create the Audio Focus Helper, if the Audio Focus feature is available (SDK 8 or above)
        if (android.os.Build.VERSION.SDK_INT >= 8)
            mAudioFocusHelper = new AudioFocusHelper(getApplicationContext(), this);
        else
            mAudioFocus = AudioFocus.Focused; // no focus feature, so we always "have" audio focus
    }

    /**
     * Called when we receive an Intent. When we receive an intent sent to us via startService(),
     * this is the method that gets called. So here we react appropriately depending on the
     * Intent's action, which specifies what is being requested of us.
     */
    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        String action = intent.getAction();
        
        if (ACTION_PLAY.equals(action)) processPlay(intent);
        else if (ACTION_POSITION_CHANGED.equals(action)) processPosition();
        else if (ACTION_PAUSE.equals(action)) processPause();
        else if (ACTION_FORWARD.equals(action)) processForward();
        else if (ACTION_REWARD.equals(action)) processReward();
        else if (ACTION_STOP.equals(action)) processStop();
        else if (ACTION_AB_REPEAT.equals(action)) processAbRepeat();
        else if (ACTION_SEEK_TO.equals(action)) processSeekTo(intent);
        else if (ACTION_REPEAT_MODE.equals(action)) processRepeat();
        else if (ACTION_PLAY_NEWS.equals(action)) processViewNews(intent);

        return START_NOT_STICKY; // Means we started the service, but don't want it to
                                 // restart in case it's killed.
    }
    
    /**
     * Request to get into to player and prepare audio file
     * @param intent
     */
    private void processViewNews(Intent intent){
    	NewsItem item = intent.getParcelableExtra(EXTRA_ITEM);
    	if (item == null) return;
    	
    	LogUtils.d(TAG, "--------------- processViewNews --------------");
    	LogUtils.d(TAG, "" + item);
    	
    	setupDataSource(item);
    	//mRetriever.find(item._id);
    }

    /**
     * Prepare audio & play audio
     */
    private void processPlay(Intent intent) {
    	mNewsItem = intent.getParcelableExtra(EXTRA_ITEM);
    	if (mNewsItem == null)
    		return;
    	
    	LogUtils.d(TAG, "NewsItem:" + mNewsItem);
    	
    	/*
    	switch (mState){
	    	case Retrieving:
	    		mWhatToPlayAfterRetrieve = null; // play a random song
	            mStartPlayingAfterRetrieve = true;
	            return;
	    	case Stopped:
	    		tryToGetAudioFocus();
	    		playNextSong(null);
	    		return;
	    	case Paused:
	    		tryToGetAudioFocus();
	    		setState(State.Playing);
	            setUpAsForeground(mCurrentAudio.title + " (playing)");
	            configAndStartMediaPlayer();
	    		return;
	    	case Playing:
	    		processPause();
	    		return;
	    	default:
	    		LogUtils.d(TAG, ">>>>>>Process Play");
	    		tryToGetAudioFocus();
	    		setUpAsForeground(mCurrentAudio.title + " (playing)");
	    		configAndStartMediaPlayer();
	    		return;
    	}
    	*/
    }
    
    private void processPosition(){
    	mTrackHandler.sendEmptyMessage(UM_TRACK_AUDIO);
    }

    private void processPause() {
        if (mState == State.Retrieving) {
            // If we are still retrieving media, clear the flag that indicates we should start
            // playing when we're ready
            mStartPlayingAfterRetrieve = false;
            return;
        }

        if (mState == State.Playing) {
            // Pause media player and cancel the 'foreground service' state.
            setState(State.Paused);
            mPlayer.pause();
            relaxResources(false); // while paused, we always retain the MediaPlayer
            giveUpAudioFocus();
        }
    }

//    private void processFastReward() {
//        if (mState == State.Playing || mState == State.Paused){
//        	tryToGetAudioFocus();
//        	playNextSong(mRetriever.prev());
//        }
//    }

//    private void processFastForward() {
//        if (mState == State.Playing || mState == State.Paused) {
//            tryToGetAudioFocus();
//            playNextSong(mRetriever.next());
//        }
//    }
    
    private void processReward() {
    	if (mState == State.Playing || mState == State.Paused)
    		mPlayer.seekTo(mPlayer.getCurrentPosition()-5000);
    }
    
    private void processForward() {
    	if (mState == State.Playing || mState == State.Paused) {
    		mPlayer.seekTo(mPlayer.getCurrentPosition()+5000);
    	}
    }

    void processStop() {
        if (mState == State.Playing || mState == State.Paused) {
        	setState(State.Stopped);

            // let go of all resources...
            relaxResources(true);
            giveUpAudioFocus();

            // service is no longer necessary. Will be started again if needed.
            stopSelf();
        }
    }

    void processAbRepeat() {
    	if (mState == State.Playing || mState == State.Paused) {
    		Intent intent = new Intent(ACTION_AB_REPEAT);
    		
    		switch (mAbIndex){
	    		case 0:
	    		case 1:
	    			int i = mAbIndex++%2;
	    			mAbPosition[i] = mPlayer.getCurrentPosition();
	    			String msg = "" + (char)('A'+i) + " set";
	    			Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();
	    			intent.putExtra(EXTRA_STATE, i);
	    			break;
	    		default:
	    			mAbPosition[0] = -1;
	    			mAbPosition[1] = -1;
	    			mAbIndex = 0; 
	    			Toast.makeText(getApplicationContext(), "AB reset", Toast.LENGTH_SHORT).show();
	    			intent.putExtra(EXTRA_STATE, 2);
	    			break;
    		}
    		sendBroadcast(intent);
    		
    	}
    }
    
    void processSeekTo(Intent intent) {
    	if (mState == State.Playing || mState == State.Paused) {
    		int position = intent.getIntExtra(EXTRA_POSITION, 0);
    		mPlayer.seekTo(position);
    	}
    }
    
    void processRepeat() {
    	//if (mState == State.Playing || mState == State.Paused) {
    		switch (mRepeatMode){
	    		case RepeatNone:
	    			mRepeatMode = RepeatMode.RepeatOne;
	    			break;
	    		case RepeatOne:
	    			mRepeatMode = RepeatMode.RepeatAll;
	    			break;
	    		case RepeatAll:
	    			mRepeatMode = RepeatMode.RepeatNone;
	    			break;
    		}
    		
    		mPlayer.setLooping(mRepeatMode == RepeatMode.RepeatOne);
    		Intent intent = new Intent(ACTION_REPEAT_MODE_CHANGED);
	    	intent.putExtra(EXTRA_STATE, mRepeatMode.ordinal());
	    	sendBroadcast(intent);
    	//}
    }
    
    /**
     * Releases resources used by the service for playback. This includes the "foreground service"
     * status and notification, the wake locks and possibly the MediaPlayer.
     *
     * @param releaseMediaPlayer Indicates whether the Media Player should also be released or not
     */
    void relaxResources(boolean releaseMediaPlayer) {
        // stop being a foreground service
        stopForeground(true);

        // stop and release the Media Player, if it's available
        if (releaseMediaPlayer && mPlayer != null) {
            mPlayer.reset();
            mPlayer.release();
            mPlayer = null;
        }

        // we can also release the Wifi lock, if we're holding it
        if (mWifiLock.isHeld()) mWifiLock.release();
    }

    void giveUpAudioFocus() {
        if (mAudioFocus == AudioFocus.Focused && mAudioFocusHelper != null
                                && mAudioFocusHelper.abandonFocus())
            mAudioFocus = AudioFocus.NoFocusNoDuck;
    }

    /**
     * Reconfigures MediaPlayer according to audio focus settings and starts/restarts it. This
     * method starts/restarts the MediaPlayer respecting the current audio focus state. So if
     * we have focus, it will play normally; if we don't have focus, it will either leave the
     * MediaPlayer paused or set it to a low volume, depending on what is allowed by the
     * current focus settings. This method assumes mPlayer != null, so if you are calling it,
     * you have to do so from a context where you are sure this is the case.
     */
    void configAndStartMediaPlayer() {
        if (mAudioFocus == AudioFocus.NoFocusNoDuck) {
            // If we don't have audio focus and can't duck, we have to pause, even if mState
            // is State.Playing. But we stay in the Playing state so that we know we have to resume
            // playback once we get the focus back.
            if (mPlayer.isPlaying()) mPlayer.pause();
            return;
        }
        else if (mAudioFocus == AudioFocus.NoFocusCanDuck)
            mPlayer.setVolume(DUCK_VOLUME, DUCK_VOLUME);  // we'll be relatively quiet
        else
            mPlayer.setVolume(1.0f, 1.0f); // we can be loud

        if (!mPlayer.isPlaying()) {
        	mPlayer.start();
        	setState(State.Playing);
        }
    }

    void processAddRequest(Intent intent) {
        // user wants to play a song directly by URL or path. The URL or path comes in the "data"
        // part of the Intent. This Intent is sent by {@link MainActivity} after the user
        // specifies the URL/path via an alert box.
        if (mState == State.Retrieving) {
            // we'll play the requested URL right after we finish retrieving
            mWhatToPlayAfterRetrieve = intent.getData();
            mStartPlayingAfterRetrieve = true;
        }
        else if (mState == State.Playing || mState == State.Paused || mState == State.Stopped) {
            LogUtils.i(TAG, "Playing from URL/path: " + intent.getData().toString());
            tryToGetAudioFocus();
            //***playNextSong(intent.getData().toString());
        }
    }
    
    private synchronized void setCurrentAudio(NewsItem value){
    	this.mNewsItem = value;
    }

    /**
     * Shortcut to making and displaying a toast. Seemed cleaner than repeating
     * this code everywhere, at least for this sample.
     */
    private void say(String message) {
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
    }

    void tryToGetAudioFocus() {
        if (mAudioFocus != AudioFocus.Focused && mAudioFocusHelper != null
                        && mAudioFocusHelper.requestFocus())
            mAudioFocus = AudioFocus.Focused;
    }
    
	public void setupDataSource(NewsItem item) throws RuntimeException{
		if (item==null || TextUtils.isEmpty(item.filePath))
			return;
		
		FileInputStream fis = null;
		try {
			
			File f = new File(item.filePath);
			if (!f.exists()){
				Toast.makeText(this, R.string.m_file_not_found, Toast.LENGTH_LONG).show();
				return;
			}
			
			fis = new FileInputStream(f);
			
			setCurrentAudio(item);
			setState(State.Preparing);
			
			mPlayer.reset();
			mPlayer.setDataSource(fis.getFD(), 0, f.length());
			mPlayer.prepareAsync();
			
		} catch (IllegalArgumentException e){
			e.printStackTrace();
			throw new RuntimeException(e);
		} catch (IllegalStateException e){
			e.printStackTrace();
			throw new RuntimeException(e);
		} catch (IOException e){
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally  {
			try { if (fis!=null) fis.close(); } catch (Exception ex){}
		}
	}

    /**
     * Starts playing the next song. If manualUrl is null, the next song will be randomly selected
     * from our Media Retriever (that is, it will be a random song in the user's device). If
     * manualUrl is non-null, then it specifies the URL or path to the song that will be played
     * next.
     */
    void playNextSong(NewsItem item) {
        
        if (item == null) {
            say("No song to play");
            return;
        }
        
        if (!item.fileExists()) {
        	Toast.makeText(this, R.string.m_file_not_found, Toast.LENGTH_LONG).show();
        	return;
        }

        setState(State.Stopped);
        relaxResources(false); //release everything except MediaPlayer

        LogUtils.d(TAG, "PlayNextSong============>" + item);
        
        try {
            if (item!= null) {
                // set the source of the media player to a manual URL or path
                //createMediaPlayerIfNeeded();
                setupDataSource(item);
                
                //mCurrentAudio = item;
                //mIsStreaming = false;
            }
            /*
            else {
                mIsStreaming = false; // playing a locally available song
                item = mRetriever.next();
                if (item == null) {
                    say("No song to play :-(");
                    return;
                }

                // set the source of the media player a a content URI
                //createMediaPlayerIfNeeded();
                setupDataSource(item);
            }
			*/

            //***setState(State.Preparing);
            setUpAsForeground(item.title + " (loading)");

            // starts preparing the media player in the background. When it's done, it will call
            // our OnPreparedListener (that is, the onPrepared() method on this class, since we set
            // the listener to 'this').
            //
            // Until the media player is prepared, we *cannot* call start() on it!
            //mPlayer.prepareAsync();
            
            // If we are streaming from the internet, we want to hold a Wifi lock, which prevents
            // the Wifi radio from going to sleep while the song is playing. If, on the other hand,
            // we are *not* streaming, we want to release the lock if we were holding it before.
            //if (mIsStreaming) mWifiLock.acquire();
            //else if 
            if (mWifiLock.isHeld()) mWifiLock.release();
        }
        catch (RuntimeException ex) {
            LogUtils.e("MusicService", "IOException playing next song: " + ex.getMessage());
            ex.printStackTrace();
        }
    }
    
    /** Called when media player is done playing current song. */
    public void onCompletion(MediaPlayer player) {
    	LogUtils.d(TAG, "************ onCompletion mRepeatMode:" + mRepeatMode);
        // The media player finished playing the current song, so we go ahead and start the next.
    	if (mRepeatMode == RepeatMode.RepeatAll){
    		Intent intent = new Intent(ACTION_NEXT_NEWS);
	    	sendBroadcast(intent);
    	}
    }

    /** Called when media player is done preparing. */
    public void onPrepared(MediaPlayer player) {
        // The media player is done preparing. That means we can start playing!
    	setState(State.Prepared);
    	updateNotification(mNewsItem.title + " (playing)");
        configAndStartMediaPlayer();
    }

    /** Updates the notification. */
   private void updateNotification(String text) {
//        PendingIntent pi = PendingIntent.getActivity(getApplicationContext(), 0,
//                new Intent(getApplicationContext(), NewsViewActivity.class),
//                PendingIntent.FLAG_UPDATE_CURRENT);
//        mNotification.setLatestEventInfo(getApplicationContext(), "RandomMusicPlayer", text, pi);
//        mNotificationManager.notify(`																																										, mNotification);
    }

    /**
     * Configures service as a foreground service. A foreground service is a service that's doing
     * something the user is actively aware of (such as playing music), and must appear to the
     * user as a notification. That's why we create the notification here.
     */
    void setUpAsForeground(String text) {
        PendingIntent pi = PendingIntent.getActivity(getApplicationContext(), 0,
                new Intent(getApplicationContext(), NewsPlayActivity.class),
                PendingIntent.FLAG_UPDATE_CURRENT);
        mNotification = new Notification();
        mNotification.tickerText = text;
        //mNotification.icon = R.drawable.ic_stat_playing;
        mNotification.flags |= Notification.FLAG_ONGOING_EVENT;
        mNotification.setLatestEventInfo(getApplicationContext(), getString(R.string.app_name), text, pi);
        startForeground(NOTIFICATION_ID, mNotification);
    }

    /**
     * Called when there's an error playing media. When this happens, the media player goes to
     * the Error state. We warn the user about the error and reset the media player.
     */
    public boolean onError(MediaPlayer mp, int what, int extra) {
        Toast.makeText(getApplicationContext(), "Media player error! Resetting.",
            Toast.LENGTH_SHORT).show();
        LogUtils.e(TAG, "Error: what=" + String.valueOf(what) + ", extra=" + String.valueOf(extra));

        setState(State.Stopped);
        relaxResources(true);
        giveUpAudioFocus();
        return true; // true indicates we handled the error
    }

    public void onGainedAudioFocus() {
        Toast.makeText(getApplicationContext(), "gained audio focus.", Toast.LENGTH_SHORT).show();
        mAudioFocus = AudioFocus.Focused;

        // restart media player with new focus settings
        if (mState == State.Playing)
            configAndStartMediaPlayer();
    }

    public void onLostAudioFocus(boolean canDuck) {
        Toast.makeText(getApplicationContext(), "lost audio focus." + (canDuck ? "can duck" :
            "no duck"), Toast.LENGTH_SHORT).show();
        mAudioFocus = canDuck ? AudioFocus.NoFocusCanDuck : AudioFocus.NoFocusNoDuck;

        // start/restart/pause media player with new focus settings
        if (mPlayer != null && mPlayer.isPlaying())
            configAndStartMediaPlayer();
    }

    public void onNewsRetrieverPrepared() {
        // Done retrieving!
    	setState(State.Stopped);
    	
    	//Intent intent = new Intent(ACTION_NEWS_RETRIEVED);
    	//sendBroadcast(intent);
    	
        // If the flag indicates we should start playing after retrieving, let's do that now.
        if (mStartPlayingAfterRetrieve) {
            tryToGetAudioFocus();
            //playNextSong(mWhatToPlayAfterRetrieve == null ?
            //        null : mWhatToPlayAfterRetrieve.toString());
        }
    }

    @Override
    public void onDestroy() {
        // Service is being killed, so make sure we release our resources
    	setState(State.Stopped);
        relaxResources(true);
        giveUpAudioFocus();
    }

    @Override
    public IBinder onBind(Intent arg0) {
        return null;
    }
}
